﻿-- ********************************************************************************
-- Data Broker Volume Control (Broker_Volume)
-- A volume control for Data Broker.
-- By: Shenton
--
-- Core.lua
-- ********************************************************************************

-- Ace libs (<3)
local A = LibStub("AceAddon-3.0"):NewAddon("BrokerVolume", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale("BrokerVolume", false);
A.L = L;

-- Addon global
_G["BrokerVolumeGlobal"] = A;

-- ********************************************************************************
-- Variables
-- ********************************************************************************

-- LUA Globals
local floor, ceil, abs = floor, ceil, abs;
local tonumber, tostring = tonumber, tostring;

-- AddOn version
A.version = GetAddOnMetadata("Broker_Volume", "Version");

-- Text colors
A.color = {};
A.color["RED"] = "|cffff3333";
A.color["GREEN"] = "|cff33ff99";
A.color["WHITE"] = "|cffffffff";
A.color["GRAY"] = "|cff999999";
A.color["WARRIOR"] = "|cffc79c6e";
A.color["RESET"] = "|r";

-- ********************************************************************************
-- Dropdown menu
-- ********************************************************************************

--- The dropdown menu structure function
-- @usage Called by ToggleDropDownMenu()
-- @param self Frame object
-- @param level Nesting level
local function DropdownMenu(self, level)
	if ( not level ) then return; end

	local info = self.info;
    local db = A.db.profile;
    local master = A:GetVolumePercent("Sound_MasterVolume");
    local effects = A:GetVolumePercent("Sound_SFXVolume");
    local music = A:GetVolumePercent("Sound_MusicVolume");
    local ambience = A:GetVolumePercent("Sound_AmbienceVolume");

	wipe(info);

	if ( level == 1 ) then
		-- Menu title
		info.isTitle = true;
		info.text = L["ADDON_NAME"];
		info.notCheckable = true;
		UIDropDownMenu_AddButton(info, level);

		-- Set options
		info.keepShownOnClick = true;
		info.hasArrow = true;
		info.disabled = nil;
		info.isTitle = nil;

        -- Master
        info.text = "   "..L["MASTER_VOLUME"];
        info.value = "MASTER";
        UIDropDownMenu_AddButton(info, level);

        -- Effects
        info.text = "   "..L["EFFECTS_VOLUME"];
        info.value = "EFFECTS";
        UIDropDownMenu_AddButton(info, level);

        -- Music
        info.text = "   "..L["MUSIC_VOLUME"];
        info.value = "MUSIC";
        UIDropDownMenu_AddButton(info, level);

        -- Ambience
        info.text = "   "..L["AMBIENCE_VOLUME"];
        info.value = "AMBIENCE";
        UIDropDownMenu_AddButton(info, level);

		-- Blank separator
		wipe(info);
		info.disabled = true;
		info.notCheckable = true;
		UIDropDownMenu_AddButton(info, level);

		-- Set options
		info.disabled = nil;

		-- Options
		info.text = "   "..L["OPTIONS"];
		info.value = "OPTIONS";
		info.hasArrow = true;
		UIDropDownMenu_AddButton(info, level);

		-- Set options
		info.keepShownOnClick = nil;
		info.hasArrow = nil;

		-- Close
		info.text = L["CLOSE"];
		info.func = function() CloseDropDownMenus(); end;
		UIDropDownMenu_AddButton(info, level);
	elseif ( level == 2 ) then
		if ( UIDROPDOWNMENU_MENU_VALUE == "MASTER" ) then -- Master
			wipe(info);

			for i=0,100,10 do
                info.text = tostring(i).."%";
                info.checked = function()
                    if ( i == master ) then
                        return true;
                    else
                        return false;
                    end
                end;
                info.func = function() SetCVar("Sound_MasterVolume", A:SetVolumeNum(i), "Doh!"); end;
                UIDropDownMenu_AddButton(info, level);
            end
		elseif ( UIDROPDOWNMENU_MENU_VALUE == "EFFECTS" ) then -- Effects
			wipe(info);

			for i=0,100,10 do
                info.text = tostring(i).."%";
                info.checked = function()
                    if ( i == effects ) then
                        return true;
                    else
                        return false;
                    end
                end;
                info.func = function() SetCVar("Sound_SFXVolume", A:SetVolumeNum(i), "Doh!"); end;
                UIDropDownMenu_AddButton(info, level);
            end
		elseif ( UIDROPDOWNMENU_MENU_VALUE == "MUSIC" ) then -- Music
			wipe(info);

			for i=0,100,10 do
                info.text = tostring(i).."%";
                info.checked = function()
                    if ( i == music ) then
                        return true;
                    else
                        return false;
                    end
                end;
                info.func = function() SetCVar("Sound_MusicVolume", A:SetVolumeNum(i), "Doh!"); end;
                UIDropDownMenu_AddButton(info, level);
            end
        elseif ( UIDROPDOWNMENU_MENU_VALUE == "AMBIENCE" ) then -- Ambience
			wipe(info);

			for i=0,100,10 do
                info.text = tostring(i).."%";
                info.checked = function()
                    if ( i == ambience ) then
                        return true;
                    else
                        return false;
                    end
                end;
                info.func = function() SetCVar("Sound_AmbienceVolume", A:SetVolumeNum(i), "Doh!"); end;
                UIDropDownMenu_AddButton(info, level);
            end
		elseif ( UIDROPDOWNMENU_MENU_VALUE == "OPTIONS" ) then -- Options
			wipe(info);

			-- Set options
			info.keepShownOnClick = true;

			-- Display spec name
			info.text = L["BLIZLIKE_MUTE"];
			info.checked = db.blizlike;
			info.func = function() db.blizlike = not db.blizlike; end;
			UIDropDownMenu_AddButton(info, level);

			-- Show/hide minimap icon
			info.text = L["SHOW_HIDE_MINIMAP"];
			info.checked = not db.minimap.hide;
			info.func = function()
				db.minimap.hide = not db.minimap.hide;
				A:ShowHideMinimap();
			end;
			UIDropDownMenu_AddButton(info, level);
		end
	end
end

-- ********************************************************************************
-- Functions
-- ********************************************************************************

--- Send a message to the chat frame with the addon name colored
-- @param text The message to display
-- @param color Bool, if true will color in red
function A:Message(text, color)
	if ( color ) then
		color = A.color["RED"];
	else
		color = A.color["GREEN"]
	end

	DEFAULT_CHAT_FRAME:AddMessage(color..L["Volume"]..": "..A.color["RESET"]..text);
end

--- Handle the slash command
function A:SlashCommand(...)
	local arg1, arg2 = ...;

	if ( arg1 == "" ) then
		A:Message(L["Command usage: /bv, /brokervolume"]);
		A:Message(L["    |cffc79c6eshow|r, show the minimap icon."]);
		A:Message(L["    |cffc79c6e<master||effects||music||ambience> <0-100>|r, will set the volume for the given category."]);
	elseif ( arg1 == "show" ) then
		A.db.profile.minimap.hide = false;
		A:ShowHideMinimap();
	elseif ( arg1 == "master" and arg2 ) then
        local volume = tonumber(arg2);

        if ( A:IsInRange(volume) ) then
            volume = A:SetVolumeNum(volume);
            SetCVar("Sound_MasterVolume", volume, "Doh!");
            A:Message(L["Master volume set to"]..": "..floor(volume * 100).."%");
        end
    elseif ( arg1 == "effects" and arg2 ) then
        local volume = tonumber(arg2);

        if ( A:IsInRange(volume) ) then
            volume = A:SetVolumeNum(volume);
            SetCVar("Sound_SFXVolume", volume, "Doh!");
            A:Message(L["Effects volume set to"]..": "..floor(volume * 100).."%");
        end
	elseif ( arg1 == "music" and arg2 ) then
        local volume = tonumber(arg2);

        if ( A:IsInRange(volume) ) then
            volume = A:SetVolumeNum(volume);
            SetCVar("Sound_MusicVolume", volume, "Doh!");
            A:Message(L["Music volume set to"]..": "..floor(volume * 100).."%");
        end
	elseif ( arg1 == "ambience" and arg2 ) then
        local volume = tonumber(arg2);

        if ( A:IsInRange(volume) ) then
            volume = A:SetVolumeNum(volume);
            SetCVar("Sound_AmbienceVolume", volume, "Doh!");
            A:Message(L["Ambience volume set to"]..": "..floor(volume * 100).."%");
        end
	else
		A:Message(L["Command usage: /bv, /brokervolume"]);
		A:Message(L["    |cffc79c6eshow|r, show the minimap icon."]);
		A:Message(L["    |cffc79c6e<master||effects||music||ambience> <0-100>|r, will set the volume for the given category."]);
	end
end

--- Round
-- @param num The number to round
-- @param idp Number of decimal
-- @return The rounded number
function A:Round(num, idp)
    local mult = 10 ^ ( idp or 0 );

    return floor(num * mult + 0.5) / mult;
end

--- Ceil decade
-- @param val The number to round
-- @return Rounded number
function A:CeilDecade(num)
    return ceil(num / 10) * 10;
end

--- Set a number to match volume cvar
-- @param val The number to set
-- @return The rounded number
function A:SetVolumeNum(num)
    num = A:CeilDecade(num);
    num = num / 100;
    num = A:Round(num, 1);

    return num;
end

--- Get the volume level in percent
-- @param cat Volume category
-- @return The volume as percent
function A:GetVolumePercent(cat)
    local volume = tonumber(GetCVar(cat));

    volume = floor(volume * 100);

    return volume;
end

--- Check if the number is between 0-100
-- @param num The number to test
function A:IsInRange(num)
    if ( num >= 0 and num <= 100) then
        return true;
    else
        A:Message(L["You must provide a number between 0 and 100."], true);
    end

    return false;
end

--- Blizzard default function Sound_ToggleMusic()
function A:ToggleMusic()
	if ( GetCVar("Sound_EnableAllSound") == "0" ) then
		ActionStatus_DisplayMessage(SOUND_DISABLED, true);
	else
        if ( GetCVar("Sound_EnableMusic") == "0" ) then
            SetCVar("Sound_EnableMusic", 1, "Doh!");
            ActionStatus_DisplayMessage(MUSIC_ENABLED, true)
        else
            SetCVar("Sound_EnableMusic", 0, "Doh!");
            ActionStatus_DisplayMessage(MUSIC_DISABLED, true)
        end
    end
end

--- Blizzard default function Sound_ToggleSound()
function A:ToggleSound()
	if ( GetCVar("Sound_EnableAllSound") == "0" ) then
		ActionStatus_DisplayMessage(SOUND_DISABLED, true);
	else
        if ( GetCVar("Sound_EnableSFX") == "0" ) then
            SetCVar("Sound_EnableSFX", 1, "Doh!");
            SetCVar("Sound_EnableAmbience", 1, "Doh!");
            ActionStatus_DisplayMessage(SOUND_EFFECTS_ENABLED, true);
        else
            SetCVar("Sound_EnableSFX", 0, "Doh!");
            SetCVar("Sound_EnableAmbience", 0, "Doh!");
            ActionStatus_DisplayMessage(SOUND_EFFECTS_DISABLED, true);
        end
    end
end

--- Toggle all soundss
function A:ToggleAll()
	if ( GetCVar("Sound_EnableAllSound") == "0" ) then
		ActionStatus_DisplayMessage(SOUND_DISABLED, true);
	else
        if ( GetCVar("Sound_EnableSFX") == "0" ) then
            SetCVar("Sound_EnableSFX", 1, "Doh!");
            SetCVar("Sound_EnableAmbience", 1, "Doh!");
            SetCVar("Sound_EnableMusic", 1, "Doh!");
            ActionStatus_DisplayMessage(SOUND_EFFECTS_ENABLED, true);
        else
            SetCVar("Sound_EnableSFX", 0, "Doh!");
            SetCVar("Sound_EnableAmbience", 0, "Doh!");
            SetCVar("Sound_EnableMusic", 0, "Doh!");
            ActionStatus_DisplayMessage(SOUND_EFFECTS_DISABLED, true);
        end
    end
end

--- Show or hide the minimap icon
function A:ShowHideMinimap()
	if ( A.db.profile.minimap.hide ) then
		A:Message(L["Minimap icon is hidden if you want to show it back use: /bv show or /brokervolume show"], true);
		A.icon:Hide("BrokerVolumeIcon");
	else
		A.icon:Show("BrokerVolumeIcon");
	end
end

--- Update Broker
function A:UpdateBroker()
    if ( GetCVar("Sound_EnableAllSound") == "0" or GetCVar("Sound_EnableSFX") == "0" ) then
        A.ldb.text = L["Mute"];
        A.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-mute";
    else
        local volume = A:GetVolumePercent("Sound_MasterVolume");

        A.ldb.text = volume.."%";
        if ( volume > 60 ) then
            A.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-max";
        elseif ( volume > 30 ) then
            A.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-medium";
        elseif ( volume > 0 ) then
            A.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-low";
        else
            A.ldb.icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-mute";
        end
    end
end

--- Callback function for event VARIABLES_LOADED
function A:VARIABLES_LOADED()
    A:UpdateBroker();
end

--- Callback function for event CVAR_UPDATE
function A:CVAR_UPDATE()
    A:UpdateBroker();
end

--- Callback function for hook Sound_MasterVolumeUp()
function A:Sound_MasterVolumeUp()
    local volume = tonumber(GetCVar("Sound_MasterVolume"));

    volume = A:Round(volume, 1);
	if ( volume and volume <= 0.9) then
		SetCVar("Sound_MasterVolume", volume + 0.1, "Doh!");
	end
end

--- Callback function for hook Sound_MasterVolumeDown()
function A:Sound_MasterVolumeDown()
    local volume = tonumber(GetCVar("Sound_MasterVolume"));

    volume = A:Round(volume, 1);
	if ( volume and volume >= 0.1 ) then
		SetCVar("Sound_MasterVolume", volume - 0.1, "Doh!");
	end
end

--- Callback function for hook Sound_ToggleSound()
function A:Sound_ToggleSound()
    if ( A.db.profile.blizlike ) then
        A:ToggleMusic();
    else
        A:ToggleAll();
    end
end

--- Callback function for hook Sound_ToggleMusic()
function A:Sound_ToggleMusic()
    if ( A.db.profile.blizlike ) then
        A:ToggleMusic();
    else
        A:ToggleAll();
    end
end

--- Return anchor points depending on cursor position
function A:GetAnchor()
    local w = GetScreenWidth();
    local x = GetCursorPosition();

    w = (w * UIParent:GetEffectiveScale()) / 2;

    if ( x > w ) then return "TOPRIGHT", "TOPLEFT"; end

    return "TOPLEFT", "TOPRIGHT";
end

--- 10% effects volume up or down
function A:EffectsVolumeUp()
    local volume = tonumber(GetCVar("Sound_SFXVolume"));

    volume = A:Round(volume, 1);
	if ( volume and volume <= 0.9) then
		SetCVar("Sound_SFXVolume", volume + 0.1, "Doh!");
	end
end
function A:EffectsVolumeDown()
    local volume = tonumber(GetCVar("Sound_SFXVolume"));

    volume = A:Round(volume, 1);
	if ( volume and volume >= 0.1 ) then
		SetCVar("Sound_SFXVolume", volume - 0.1, "Doh!");
	end
end

--- 10% music volume up or down
function A:MusicVolumeUp()
    local volume = tonumber(GetCVar("Sound_MusicVolume"));

    volume = A:Round(volume, 1);
	if ( volume and volume <= 0.9) then
		SetCVar("Sound_MusicVolume", volume + 0.1, "Doh!");
	end
end
function A:MusicVolumeDown()
    local volume = tonumber(GetCVar("Sound_MusicVolume"));

    volume = A:Round(volume, 1);
	if ( volume and volume >= 0.1 ) then
		SetCVar("Sound_MusicVolume", volume - 0.1, "Doh!");
	end
end

--- 10% ambience volume up or down
function A:AmbienceVolumeUp()
    local volume = tonumber(GetCVar("Sound_AmbienceVolume"));

    volume = A:Round(volume, 1);
	if ( volume and volume <= 0.9) then
		SetCVar("Sound_AmbienceVolume", volume + 0.1, "Doh!");
	end
end
function A:AmbienceVolumeDown()
    local volume = tonumber(GetCVar("Sound_AmbienceVolume"));

    volume = A:Round(volume, 1);
	if ( volume and volume >= 0.1 ) then
		SetCVar("Sound_AmbienceVolume", volume - 0.1, "Doh!");
	end
end

-- ********************************************************************************
-- Configuration DB
-- ********************************************************************************

--- Default configuration table for AceDB
local aceDefaultDB =
{
	profile =
	{
        blizlike = nil,
		minimap =
		{
			hide = 1,
		},
	},
};

-- ********************************************************************************
-- Main
-- ********************************************************************************

--- AceAddon callback
-- Called after the addon is fully loaded
function A:OnInitialize()
    -- Config db
	A.db = LibStub("AceDB-3.0"):New("BrokerVolumeDB", aceDefaultDB);

	-- LDB
	A.ldb = LibStub("LibDataBroker-1.1"):NewDataObject("BrokerVolumeObject",
	{
		type = "data source",
		text = "",
		label = L["Volume"],
		icon = "Interface\\AddOns\\Broker_Volume\\Graphics\\sound-mute",
		tocname = "Broker_Volume",
        OnMouseWheel = function(self, delta)
            if ( delta > 0 ) then
                A:Sound_MasterVolumeUp();
            else
                A:Sound_MasterVolumeDown();
            end
        end,
		OnClick = function(self, button)
			if (button == "LeftButton") then
				if ( A.db.profile.blizlike ) then
                    A:ToggleMusic();
                else
                    A:ToggleAll();
                end
			elseif ( button == "RightButton" ) then
				-- if ( A.menuFrame.initialize ~= DropdownMenu ) then
					-- A.menuFrame.initialize = DropdownMenu;
				-- end
                -- CloseDropDownMenus();
				-- ToggleDropDownMenu(1, nil, A.menuFrame, self:GetName(), 0, 0);
				-- GameTooltip:Hide();
                if ( not A.slidersFrame ) then A:CreateSlidersFrame(); end
                local point, relativePoint = A:GetAnchor();
                A.slidersFrame:ClearAllPoints()
                A.slidersFrame:SetPoint(point, self, relativePoint, 0, 0);
                A.slidersFrame:Show();
			end
		end,
		OnTooltipShow = function(tooltip)
			tooltip:AddDoubleLine(A.color["WHITE"]..L["Volume"], A.color["GREEN"].." v"..A.version);
			tooltip:AddLine(" ");

			tooltip:AddLine(L["Master volume"]..": "..A:GetVolumePercent("Sound_MasterVolume").."%");
            tooltip:AddLine(L["Effects volume"]..": "..A:GetVolumePercent("Sound_SFXVolume").."%");
            tooltip:AddLine(L["Music volume"]..": "..A:GetVolumePercent("Sound_MusicVolume").."%");
            tooltip:AddLine(L["Ambience volume"]..": "..A:GetVolumePercent("Sound_AmbienceVolume").."%");

			tooltip:AddLine(" ");
			tooltip:AddLine(L["|cffc79c6eLeft-Click: |cff33ff99Mute sound\n|cffc79c6eRight-Click: |cff33ff99Display the configuration menu"]);
		end
	});

	-- LDBIcon
	LibStub("LibDBIcon-1.0"):Register("BrokerVolumeIcon", A.ldb, A.db.profile.minimap);

    -- Menu frame & table
	A.menuFrame = CreateFrame("Frame", "BrokerVolumeMenuFrame");
	A.menuFrame.displayMode = "MENU";
    A.menuFrame.info = {};

	-- Events
	A:RegisterEvent("VARIABLES_LOADED");
	A:RegisterEvent("CVAR_UPDATE");

    -- Hooks
    A:RawHook("Sound_MasterVolumeUp", true);
    A:RawHook("Sound_MasterVolumeDown", true);
    A:RawHook("Sound_ToggleSound", true);
    A:RawHook("Sound_ToggleMusic", true);

    -- Commands
    A:RegisterChatCommand("brokervolume", "SlashCommand");
	A:RegisterChatCommand("bv", "SlashCommand");

    A:CreateSlidersFrame();
end
